/* initial.c - functions to get things set up and initialized */

#include "external.h"
#include <math.h>

initialize ()
/* Initialization Coordinator */
{
  /* get basic problem values from input file */
  initdata ();

  /* malloc space for global data structures */
  initmalloc ();

  /* initialize application dependent variables */
  app_init ();

  /* initialize random number generator */
  srand48 (time (0));
  randomize ();
  initrandomnormaldeviate ();

  /* initialize global counters/values */
  bestfit.fitness = 10000000.0;	/* Initialize to a large number */
  bestfit.generation = 0;

  /* initialize the populations and report statistics */
  initpop ();
  statistics (oldpop);
  initreport ();
}

initdata ()
/* data inquiry and setup */
{
  char answer[2];

  if (numfiles == 0)
    {
      fprintf (outfp, "\n ------- EP Data Entry and Initialization -------\n");
      fprintf (outfp, " Enter the population size ------------> ");
    }
  fscanf (infp, "%d", &popsize);


  if (numfiles == 0)
    fprintf (outfp, " Enter chromosome length --------------> ");
  fscanf (infp, "%d", &lchrom);

  if (numfiles == 0)
    fprintf (outfp, " Enter maximum number of generations --> ");
  fscanf (infp, "%d", &maxgen);

  if (numfiles == 0)
    fprintf (outfp, " Enter the range for initializing the population --> ");
  fscanf (infp, "%lf", &inirange);


  if (numfiles == 0)
    fprintf (outfp, " Enter the Scaling Factor for mutation  --> ");
  fscanf (infp, "%lf", &scalefactor);

  if (numfiles == 0)
    fprintf (outfp, " Enter the offset for mutation ---> ");
  fscanf (infp, "%lf", &offset);

  /* any application-dependent global input */
  app_data ();
}

initpop ()
/* Initialize a population and the step sizes at random */
{
  int j, k;
  float rndreal ();

  for (j = 0; j < popsize; j++)
    {
      for (k = 0; k < lchrom; k++)
	{
	  oldpop[j].chrom[k] = rndreal (-1.0 * inirange, 1.0 * inirange);
	}
      objfunc (&(oldpop[j]));	/* Evaluate initial fitness */
    }
}

initreport ()
/* Initial report */
{
  fprintf (outfp, " EP  Parameters\n");
  fprintf (outfp, " -------------------------------------------------\n");
  fprintf (outfp, " Total Population size              =   %d\n", popsize);
  fprintf (outfp, " Chromosome length (lchrom)         =   %d\n", lchrom);
  fprintf (outfp, " Maximum # of generations (maxgen)  =   %d\n", maxgen);

  /* application dependant report */
  app_initreport ();

  fflush (outfp);
}
