/* memory.c - memory management routines for ep code */

#include "external.h"

initmalloc ()
/* memory allocation of space for global data structures */
{
  unsigned nbytes;
  char *malloc ();
  int j;

  /* memory for old and new populations of individuals */
  nbytes = popsize * sizeof (struct individual);
  if ((oldpop = (struct individual *) malloc (nbytes)) == NULL)
    nomemory (stderr, "oldpop");

  if ((newpop = (struct individual *) malloc (nbytes)) == NULL)
    nomemory (stderr, "newpop");

  /* Memory for tempop which is twice the size of old and new pop */
  nbytes = 2 * popsize * sizeof (struct individual);
  if ((temppop = (struct individual *) malloc (nbytes)) == NULL)
    nomemory (stderr, "temppop");

  /* memory for chromosome strings in populations */
  nbytes = lchrom * sizeof (double);
  for (j = 0; j < popsize; j++)
    {
      if ((oldpop[j].chrom = (double *) malloc (nbytes)) == NULL)
	nomemory (stderr, "oldpop chromosomes");

      if ((newpop[j].chrom = (double *) malloc (nbytes)) == NULL)
	nomemory (stderr, "newpop chromosomes");
    }

  if ((bestfit.chrom = (double *) malloc (nbytes)) == NULL)
    nomemory (stderr, "bestfit chromosome");

  for (j = 0; j < 2 * popsize; j++)
    {
      if ((temppop[j].chrom = (double *) malloc (nbytes)) == NULL)
	nomemory (stderr, "temppop chromosomes");
    }

  /* call to application-specific malloc() routines   */
  /* can be used to malloc memory for utility pointer */
  app_malloc ();
}


freeall ()
/* A routine to free all the space dynamically allocated in initspace() */
{
  int i;

  for (i = 0; i < popsize; i++)
    {
      free (oldpop[i].chrom);
      free (newpop[i].chrom);
    }
  free (bestfit.chrom);

  for (i = 0; i < 2 * popsize; i++)
    {
      free (temppop[i].chrom);
    }
  free (oldpop);
  free (newpop);
  free (temppop);
  free (bestfit);

  /* call to application-specific free() routines   */
  /* can be used to free memory for utility pointer */
  app_free ();
}

nomemory (fp, string)
     FILE *fp;
     char *string;
{
  fprintf (fp, "malloc: out of memory making %s!!\n", string);
  exit (-1);
}
